@@ -4,24 +4,23 @@ from mypy.types import (
     Type, Instance, CallableType, TypeVisitor, UnboundType, AnyType,
     NoneType, TypeVarType, Overloaded, TupleType, TypedDictType, UnionType,
     ErasedType, PartialType, DeletedType, UninhabitedType, TypeType, TypeVarId,
-    FunctionLike, TypeVarDef, LiteralType, get_proper_type, ProperType
-)
+    FunctionLike, TypeVarDef, LiteralType, get_proper_type, ProperType,
+    TypeAliasType)
 
 
-def expand_type(typ: Type, env: Mapping[TypeVarId, Type]) -> ProperType:
+def expand_type(typ: Type, env: Mapping[TypeVarId, Type]) -> Type:
     """Substitute any type variable references in a type given by a type
     environment.
     """
-
+    # TODO: use an overloaded signature? (ProperType stays proper after expansion.)
     return typ.accept(ExpandTypeVisitor(env))
 
 
-def expand_type_by_instance(typ: Type, instance: Instance) -> ProperType:
+def expand_type_by_instance(typ: Type, instance: Instance) -> Type:
     """Substitute type variables in type using values from an Instance.
     Type variables are considered to be bound by the class declaration."""
-    typ = get_proper_type(typ)
-
-    if instance.args == []:
+    # TODO: use an overloaded signature? (ProperType stays proper after expansion.)
+    if not instance.args:
         return typ
     else:
         variables = {}  # type: Dict[TypeVarId, Type]
@@ -53,7 +52,7 @@ def freshen_function_type_vars(callee: F) -> F:
         return cast(F, fresh_overload)
 
 
-class ExpandTypeVisitor(TypeVisitor[ProperType]):
+class ExpandTypeVisitor(TypeVisitor[Type]):
     """Visitor that substitutes type variables with values."""
 
     variables = None  # type: Mapping[TypeVarId, Type]  # TypeVar id -> TypeVar value
@@ -61,30 +60,30 @@ class ExpandTypeVisitor(TypeVisitor[ProperType]):
     def __init__(self, variables: Mapping[TypeVarId, Type]) -> None:
         self.variables = variables
 
-    def visit_unbound_type(self, t: UnboundType) -> ProperType:
+    def visit_unbound_type(self, t: UnboundType) -> Type:
         return t
 
-    def visit_any(self, t: AnyType) -> ProperType:
+    def visit_any(self, t: AnyType) -> Type:
         return t
 
-    def visit_none_type(self, t: NoneType) -> ProperType:
+    def visit_none_type(self, t: NoneType) -> Type:
         return t
 
-    def visit_uninhabited_type(self, t: UninhabitedType) -> ProperType:
+    def visit_uninhabited_type(self, t: UninhabitedType) -> Type:
         return t
 
-    def visit_deleted_type(self, t: DeletedType) -> ProperType:
+    def visit_deleted_type(self, t: DeletedType) -> Type:
         return t
 
-    def visit_erased_type(self, t: ErasedType) -> ProperType:
+    def visit_erased_type(self, t: ErasedType) -> Type:
         # Should not get here.
         raise RuntimeError()
 
-    def visit_instance(self, t: Instance) -> ProperType:
+    def visit_instance(self, t: Instance) -> Type:
         args = self.expand_types(t.args)
         return Instance(t.type, args, t.line, t.column)
 
-    def visit_type_var(self, t: TypeVarType) -> ProperType:
+    def visit_type_var(self, t: TypeVarType) -> Type:
         repl = get_proper_type(self.variables.get(t.id, t))
         if isinstance(repl, Instance):
             inst = repl
@@ -94,44 +93,50 @@ class ExpandTypeVisitor(TypeVisitor[ProperType]):
         else:
             return repl
 
-    def visit_callable_type(self, t: CallableType) -> ProperType:
+    def visit_callable_type(self, t: CallableType) -> Type:
         return t.copy_modified(arg_types=self.expand_types(t.arg_types),
                                ret_type=t.ret_type.accept(self))
 
-    def visit_overloaded(self, t: Overloaded) -> ProperType:
+    def visit_overloaded(self, t: Overloaded) -> Type:
         items = []  # type: List[CallableType]
         for item in t.items():
             new_item = item.accept(self)
+            assert isinstance(new_item, ProperType)
             assert isinstance(new_item, CallableType)
             items.append(new_item)
         return Overloaded(items)
 
-    def visit_tuple_type(self, t: TupleType) -> ProperType:
+    def visit_tuple_type(self, t: TupleType) -> Type:
         return t.copy_modified(items=self.expand_types(t.items))
 
-    def visit_typeddict_type(self, t: TypedDictType) -> ProperType:
+    def visit_typeddict_type(self, t: TypedDictType) -> Type:
         return t.copy_modified(item_types=self.expand_types(t.items.values()))
 
-    def visit_literal_type(self, t: LiteralType) -> ProperType:
+    def visit_literal_type(self, t: LiteralType) -> Type:
         # TODO: Verify this implementation is correct
         return t
 
-    def visit_union_type(self, t: UnionType) -> ProperType:
+    def visit_union_type(self, t: UnionType) -> Type:
         # After substituting for type variables in t.items,
         # some of the resulting types might be subtypes of others.
         from mypy.typeops import make_simplified_union  # asdf
         return make_simplified_union(self.expand_types(t.items), t.line, t.column)
 
-    def visit_partial_type(self, t: PartialType) -> ProperType:
+    def visit_partial_type(self, t: PartialType) -> Type:
         return t
 
-    def visit_type_type(self, t: TypeType) -> ProperType:
+    def visit_type_type(self, t: TypeType) -> Type:
         # TODO: Verify that the new item type is valid (instance or
         # union of instances or Any).  Sadly we can't report errors
         # here yet.
         item = t.item.accept(self)
         return TypeType.make_normalized(item)
 
+    def visit_type_alias_type(self, t: TypeAliasType) -> Type:
+        # Target of the type alias cannot contain type variables,
+        # so we just expand the arguments.
+        return t.copy_modified(args=self.expand_types(t.args))
+
     def expand_types(self, types: Iterable[Type]) -> List[Type]:
         a = []  # type: List[Type]
         for t in types:
