@@ -6,14 +6,14 @@ from typing import AbstractSet, Dict, Iterable, List, NamedTuple, Optional, Set,
 
 FileData = NamedTuple('FileData', [('st_mtime', float),
                                    ('st_size', int),
-                                   ('md5', str)])
+                                   ('hash', str)])
 
 
 class FileSystemWatcher:
     """Watcher for file system changes among specific paths.
 
     All file system access is performed using FileSystemCache. We
-    detect changed files by stat()ing them all and comparing md5 hashes
+    detect changed files by stat()ing them all and comparing hashes
     of potentially changed files. If a file has both size and mtime
     unmodified, the file is assumed to be unchanged.
 
@@ -54,8 +54,8 @@ class FileSystemWatcher:
 
     def _update(self, path: str) -> None:
         st = self.fs.stat(path)
-        md5 = self.fs.md5(path)
-        self._file_data[path] = FileData(st.st_mtime, st.st_size, md5)
+        hash_digest = self.fs.hash_digest(path)
+        self._file_data[path] = FileData(st.st_mtime, st.st_size, hash_digest)
 
     def _find_changed(self, paths: Iterable[str]) -> AbstractSet[str]:
         changed = set()
@@ -76,10 +76,10 @@ class FileSystemWatcher:
                 # Round mtimes down, to match the mtimes we write to meta files
                 elif st.st_size != old.st_size or int(st.st_mtime) != int(old.st_mtime):
                     # Only look for changes if size or mtime has changed as an
-                    # optimization, since calculating md5 is expensive.
-                    new_md5 = self.fs.md5(path)
+                    # optimization, since calculating hash is expensive.
+                    new_hash = self.fs.hash_digest(path)
                     self._update(path)
-                    if st.st_size != old.st_size or new_md5 != old.md5:
+                    if st.st_size != old.st_size or new_hash != old.hash:
                         # Changed file.
                         changed.add(path)
         return changed
