@@ -49,10 +49,18 @@ class ModuleNotFoundReason(Enum):
     # corresponding *-stubs package.
     FOUND_WITHOUT_TYPE_HINTS = 1
 
+    # The module was not found in the current working directory, but
+    # was able to be found in the parent directory.
+    WRONG_WORKING_DIRECTORY = 2
+
     def error_message_templates(self) -> Tuple[str, str]:
         if self is ModuleNotFoundReason.NOT_FOUND:
             msg = "Cannot find implementation or library stub for module named '{}'"
             note = "See https://mypy.readthedocs.io/en/latest/running_mypy.html#missing-imports"
+        elif self is ModuleNotFoundReason.WRONG_WORKING_DIRECTORY:
+            msg = "Cannot find implementation or library stub for module named '{}'"
+            note = ("You may be running mypy in a subpackage, "
+                    "mypy should be run on the package root")
         elif self is ModuleNotFoundReason.FOUND_WITHOUT_TYPE_HINTS:
             msg = "Skipping analyzing '{}': found module but no type hints or library stubs"
             note = "See https://mypy.readthedocs.io/en/latest/running_mypy.html#missing-imports"
@@ -166,6 +174,9 @@ class FindModuleCache:
         """Return the path of the module source file or why it wasn't found."""
         if id not in self.results:
             self.results[id] = self._find_module(id)
+            if (self.results[id] is ModuleNotFoundReason.NOT_FOUND
+                    and self._can_find_module_in_parent_dir(id)):
+                self.results[id] = ModuleNotFoundReason.WRONG_WORKING_DIRECTORY
         return self.results[id]
 
     def _find_module_non_stub_helper(self, components: List[str],
@@ -192,6 +203,20 @@ class FindModuleCache:
                 self.ns_ancestors[pkg_id] = path
             path = os.path.dirname(path)
 
+    def _can_find_module_in_parent_dir(self, id: str) -> bool:
+        """Test if a module can be found by checking the parent directories
+        of the current working directory.
+        """
+        working_dir = os.getcwd()
+        parent_search = FindModuleCache(SearchPaths((), (), (), ()))
+        while any(file.endswith(("__init__.py", "__init__.pyi"))
+                  for file in os.listdir(working_dir)):
+            working_dir = os.path.dirname(working_dir)
+            parent_search.search_paths = SearchPaths((working_dir,), (), (), ())
+            if not isinstance(parent_search._find_module(id), ModuleNotFoundReason):
+                return True
+        return False
+
     def _find_module(self, id: str) -> ModuleSearchResult:
         fscache = self.fscache
 
@@ -448,7 +473,7 @@ def default_lib_path(data_dir: str,
 
 
 @functools.lru_cache(maxsize=None)
-def get_site_packages_dirs(python_executable: Optional[str]) -> Tuple[List[str], List[str]]:
+def get_site_packages_dirs(python_executable: str) -> Tuple[List[str], List[str]]:
     """Find package directories for given python.
 
     This runs a subprocess call, which generates a list of the egg directories, and the site
@@ -461,8 +486,6 @@ def get_site_packages_dirs(python_executable: Optional[str]) -> Tuple[List[str],
         else:
             return os.path.join(root, os.path.normpath(path))
 
-    if python_executable is None:
-        return [], []
     if python_executable == sys.executable:
         # Use running Python's package dirs
         site_packages = sitepkgs.getsitepackages()
@@ -543,7 +566,11 @@ def compute_search_paths(sources: List[BuildSource],
     if alt_lib_path:
         mypypath.insert(0, alt_lib_path)
 
-    egg_dirs, site_packages = get_site_packages_dirs(options.python_executable)
+    if options.python_executable is None:
+        egg_dirs = []  # type: List[str]
+        site_packages = []  # type: List[str]
+    else:
+        egg_dirs, site_packages = get_site_packages_dirs(options.python_executable)
     for site_dir in site_packages:
         assert site_dir not in lib_path
         if (site_dir in mypypath or
