@@ -44,7 +44,7 @@ def get_package_properties(package_id: str) -> ModuleProperties:
     try:
         package = importlib.import_module(package_id)
     except BaseException as e:
-        raise InspectError(str(e))
+        raise InspectError(str(e)) from e
     name = getattr(package, '__name__', None)
     file = getattr(package, '__file__', None)
     path = getattr(package, '__path__', None)  # type: Optional[List[str]]
