@@ -8,7 +8,8 @@ from typing import Tuple, List, Dict, Mapping, Optional, Union, cast, Iterator
 from typing_extensions import Final
 
 from mypy.types import (
-    Type, TupleType, AnyType, TypeOfAny, TypeVarDef, CallableType, TypeType, TypeVarType
+    Type, TupleType, AnyType, TypeOfAny, TypeVarDef, CallableType, TypeType, TypeVarType,
+    UnboundType,
 )
 from mypy.semanal_shared import (
     SemanticAnalyzerInterface, set_callable_name, calculate_tuple_fallback, PRIORITY_FALLBACKS
@@ -178,14 +179,20 @@ class NamedTupleAnalyzer:
             info = self.build_namedtuple_typeinfo(name, [], [], {}, node.line)
             self.store_namedtuple_info(info, name, call, is_typed)
             return True, info
-        name = cast(Union[StrExpr, BytesExpr, UnicodeExpr], call.args[0]).value
-        if name != var_name or is_func_scope:
-            # There are three special cases where need to give it a unique name derived
+
+        # We use the variable name as the class name if it exists. If
+        # it doesn't, we use the name passed as an argument. We prefer
+        # the variable name because it should be unique inside a
+        # module, and so we don't need to disambiguate it with a line
+        # number.
+        if var_name:
+            name = var_name
+        else:
+            name = cast(Union[StrExpr, BytesExpr, UnicodeExpr], call.args[0]).value
+
+        if var_name is None or is_func_scope:
+            # There are two special cases where need to give it a unique name derived
             # from the line number:
-            #   * There is a name mismatch with l.h.s., therefore we need to disambiguate
-            #     situations like:
-            #         A = NamedTuple('Same', [('x', int)])
-            #         B = NamedTuple('Same', [('y', str)])
             #   * This is a base class expression, since it often matches the class name:
             #         class NT(NamedTuple('NT', [...])):
             #             ...
@@ -334,6 +341,9 @@ class NamedTupleAnalyzer:
                 except TypeTranslationError:
                     return self.fail_namedtuple_arg('Invalid field type', type_node)
                 analyzed = self.api.anal_type(type)
+                # Workaround #4987 and avoid introducing a bogus UnboundType
+                if isinstance(analyzed, UnboundType):
+                    analyzed = AnyType(TypeOfAny.from_error)
                 # These should be all known, otherwise we would defer in visit_assignment_stmt().
                 if analyzed is None:
                     return None
@@ -372,6 +382,8 @@ class NamedTupleAnalyzer:
         tuple_base = TupleType(types, fallback)
         info.tuple_type = tuple_base
         info.line = line
+        # For use by mypyc.
+        info.metadata['namedtuple'] = {'fields': items.copy()}
 
         # We can't calculate the complete fallback type until after semantic
         # analysis, since otherwise base classes might be incomplete. Postpone a
