@@ -10,6 +10,7 @@ import subprocess
 import sys
 
 from typing import List
+from typing import Optional
 
 from mypy.test.config import test_temp_dir, PREFIX
 from mypy.test.data import DataDrivenTestCase, DataSuite
@@ -45,6 +46,7 @@ def test_python_cmdline(testcase: DataDrivenTestCase, step: int) -> None:
         for s in testcase.input:
             file.write('{}\n'.format(s))
     args = parse_args(testcase.input[0])
+    custom_cwd = parse_cwd(testcase.input[1]) if len(testcase.input) > 1 else None
     args.append('--show-traceback')
     args.append('--no-site-packages')
     if '--error-summary' not in args:
@@ -56,7 +58,10 @@ def test_python_cmdline(testcase: DataDrivenTestCase, step: int) -> None:
     process = subprocess.Popen(fixed + args,
                                stdout=subprocess.PIPE,
                                stderr=subprocess.PIPE,
-                               cwd=test_temp_dir,
+                               cwd=os.path.join(
+                                   test_temp_dir,
+                                   custom_cwd or ""
+                                   ),
                                env=env)
     outb, errb = process.communicate()
     result = process.returncode
@@ -89,6 +94,8 @@ def test_python_cmdline(testcase: DataDrivenTestCase, step: int) -> None:
         if obvious_result != result:
             out.append('== Return code: {}'.format(result))
         expected_out = testcase.output if step == 1 else testcase.output2[step]
+        # Strip "tmp/" out of the test so that # E: works...
+        expected_out = [s.replace("tmp" + os.sep, "") for s in expected_out]
         assert_string_arrays_equal(expected_out, out,
                                    'Invalid output ({}, line {}){}'.format(
                                        testcase.file, testcase.line,
@@ -110,3 +117,18 @@ def parse_args(line: str) -> List[str]:
     if not m:
         return []  # No args; mypy will spit out an error.
     return m.group(1).split()
+
+
+def parse_cwd(line: str) -> Optional[str]:
+    """Parse the second line of the program for the command line.
+
+    This should have the form
+
+      # cwd: <directory>
+
+    For example:
+
+      # cwd: main/subdir
+    """
+    m = re.match('# cwd: (.*)$', line)
+    return m.group(1) if m else None
