@@ -8,7 +8,7 @@ from mypy.nodes import (
 )
 from mypyc.ir.ops import (
     Value, Call, LoadErrorValue, LoadStatic, InitStatic, TupleSet, SetAttr, Return,
-    BasicBlock, Branch, MethodCall, NAMESPACE_TYPE
+    BasicBlock, Branch, MethodCall, NAMESPACE_TYPE, LoadAddress
 )
 from mypyc.ir.rtypes import (
     RInstance, object_rprimitive, bool_rprimitive, dict_rprimitive, is_optional_type,
@@ -19,10 +19,9 @@ from mypyc.ir.class_ir import ClassIR, NonExtClassInfo
 from mypyc.primitives.generic_ops import py_setattr_op, py_hasattr_op
 from mypyc.primitives.misc_ops import (
     dataclass_sleight_of_hand, pytype_from_template_op, py_calc_meta_op, type_object_op,
-    not_implemented_op, true_op
+    not_implemented_op
 )
-from mypyc.primitives.dict_ops import dict_set_item_op, new_dict_op
-from mypyc.primitives.tuple_ops import new_tuple_op
+from mypyc.primitives.dict_ops import dict_set_item_op, dict_new_op
 from mypyc.common import SELF_NAME
 from mypyc.irbuild.util import (
     is_dataclass_decorator, get_func_def, is_dataclass, is_constant, add_self_to_env
@@ -73,7 +72,7 @@ def transform_class_def(builder: IRBuilder, cdef: ClassDef) -> None:
         # We populate __annotations__ for non-extension classes
         # because dataclasses uses it to determine which attributes to compute on.
         # TODO: Maybe generate more precise types for annotations
-        non_ext_anns = builder.primitive_op(new_dict_op, [], cdef.line)
+        non_ext_anns = builder.call_c(dict_new_op, [], cdef.line)
         non_ext = NonExtClassInfo(non_ext_dict, non_ext_bases, non_ext_anns, non_ext_metaclass)
         dataclass_non_ext = None
         type_obj = None
@@ -124,7 +123,7 @@ def transform_class_def(builder: IRBuilder, cdef: ClassDef) -> None:
                 continue
             typ = builder.load_native_type_object(cdef.fullname)
             value = builder.accept(stmt.rvalue)
-            builder.primitive_op(
+            builder.call_c(
                 py_setattr_op, [typ, builder.load_static_unicode(lvalue.name), value], stmt.line)
             if builder.non_function_scope() and stmt.is_final_def:
                 builder.init_final_static(lvalue, value, cdef.name)
@@ -149,12 +148,12 @@ def transform_class_def(builder: IRBuilder, cdef: ClassDef) -> None:
         builder.add(InitStatic(non_ext_class, cdef.name, builder.module_name, NAMESPACE_TYPE))
 
         # Add the non-extension class to the dict
-        builder.primitive_op(dict_set_item_op,
-                          [
-                              builder.load_globals_dict(),
-                              builder.load_static_unicode(cdef.name),
-                              non_ext_class
-                          ], cdef.line)
+        builder.call_c(dict_set_item_op,
+                       [
+                           builder.load_globals_dict(),
+                           builder.load_static_unicode(cdef.name),
+                           non_ext_class
+                       ], cdef.line)
 
         # Cache any cachable class attributes
         cache_class_attrs(builder, attrs_to_cache, cdef)
@@ -165,7 +164,7 @@ def allocate_class(builder: IRBuilder, cdef: ClassDef) -> Value:
     base_exprs = cdef.base_type_exprs + cdef.removed_base_type_exprs
     if base_exprs:
         bases = [builder.accept(x) for x in base_exprs]
-        tp_bases = builder.primitive_op(new_tuple_op, bases, cdef.line)
+        tp_bases = builder.new_tuple(bases, cdef.line)
     else:
         tp_bases = builder.add(LoadErrorValue(object_rprimitive, is_borrowed=True))
     modname = builder.load_static_unicode(builder.module_name)
@@ -182,7 +181,7 @@ def allocate_class(builder: IRBuilder, cdef: ClassDef) -> Value:
                      None, builder.module_name,
                      FuncSignature([], bool_rprimitive)), [], -1))
     # Populate a '__mypyc_attrs__' field containing the list of attrs
-    builder.primitive_op(py_setattr_op, [
+    builder.call_c(py_setattr_op, [
         tp, builder.load_static_unicode('__mypyc_attrs__'),
         create_mypyc_attrs_tuple(builder, builder.mapper.type_to_ir[cdef.info], cdef.line)],
         cdef.line)
@@ -191,12 +190,12 @@ def allocate_class(builder: IRBuilder, cdef: ClassDef) -> Value:
     builder.add(InitStatic(tp, cdef.name, builder.module_name, NAMESPACE_TYPE))
 
     # Add it to the dict
-    builder.primitive_op(dict_set_item_op,
-                      [
-                          builder.load_globals_dict(),
-                          builder.load_static_unicode(cdef.name),
-                          tp,
-                      ], cdef.line)
+    builder.call_c(dict_set_item_op,
+                [
+                    builder.load_globals_dict(),
+                    builder.load_static_unicode(cdef.name),
+                    tp,
+                ], cdef.line)
 
     return tp
 
@@ -219,7 +218,7 @@ def populate_non_ext_bases(builder: IRBuilder, cdef: ClassDef) -> Value:
 
         base = builder.load_global_str(cls.name, cdef.line)
         bases.append(base)
-    return builder.primitive_op(new_tuple_op, bases, cdef.line)
+    return builder.new_tuple(bases, cdef.line)
 
 
 def find_non_ext_metaclass(builder: IRBuilder, cdef: ClassDef, bases: Value) -> Value:
@@ -227,7 +226,8 @@ def find_non_ext_metaclass(builder: IRBuilder, cdef: ClassDef, bases: Value) ->
     if cdef.metaclass:
         declared_metaclass = builder.accept(cdef.metaclass)
     else:
-        declared_metaclass = builder.primitive_op(type_object_op, [], cdef.line)
+        declared_metaclass = builder.add(LoadAddress(type_object_op.type,
+                                                     type_object_op.src, cdef.line))
 
     return builder.primitive_op(py_calc_meta_op, [declared_metaclass, bases], cdef.line)
 
@@ -241,9 +241,9 @@ def setup_non_ext_dict(builder: IRBuilder,
     This class dictionary is passed to the metaclass constructor.
     """
     # Check if the metaclass defines a __prepare__ method, and if so, call it.
-    has_prepare = builder.primitive_op(py_hasattr_op,
-                                    [metaclass,
-                                    builder.load_static_unicode('__prepare__')], cdef.line)
+    has_prepare = builder.call_c(py_hasattr_op,
+                                [metaclass,
+                                builder.load_static_unicode('__prepare__')], cdef.line)
 
     non_ext_dict = builder.alloc_temp(dict_rprimitive)
 
@@ -258,7 +258,7 @@ def setup_non_ext_dict(builder: IRBuilder,
     builder.goto(exit_block)
 
     builder.activate_block(false_block)
-    builder.assign(non_ext_dict, builder.primitive_op(new_dict_op, [], cdef.line), cdef.line)
+    builder.assign(non_ext_dict, builder.call_c(dict_new_op, [], cdef.line), cdef.line)
     builder.goto(exit_block)
     builder.activate_block(exit_block)
 
@@ -279,8 +279,8 @@ def add_non_ext_class_attr(builder: IRBuilder,
     # which attributes to compute on.
     # TODO: Maybe generate more precise types for annotations
     key = builder.load_static_unicode(lvalue.name)
-    typ = builder.primitive_op(type_object_op, [], stmt.line)
-    builder.primitive_op(dict_set_item_op, [non_ext.anns, key, typ], stmt.line)
+    typ = builder.add(LoadAddress(type_object_op.type, type_object_op.src, stmt.line))
+    builder.call_c(dict_set_item_op, [non_ext.anns, key, typ], stmt.line)
 
     # Only add the attribute to the __dict__ if the assignment is of the form:
     # x: type = value (don't add attributes of the form 'x: type' to the __dict__).
@@ -349,7 +349,7 @@ def generate_attr_defaults(builder: IRBuilder, cdef: ClassDef) -> None:
         val = builder.coerce(builder.accept(stmt.rvalue), attr_type, stmt.line)
         builder.add(SetAttr(self_var, lvalue.name, val, -1))
 
-    builder.add(Return(builder.primitive_op(true_op, [], -1)))
+    builder.add(Return(builder.true()))
 
     blocks, env, ret_type, _ = builder.leave()
     ir = FuncIR(
@@ -393,7 +393,8 @@ def gen_glue_ne_method(builder: IRBuilder, cls: ClassIR, line: int) -> FuncIR:
     # If __eq__ returns NotImplemented, then __ne__ should also
     not_implemented_block, regular_block = BasicBlock(), BasicBlock()
     eqval = builder.add(MethodCall(args[0], '__eq__', [args[1]], line))
-    not_implemented = builder.primitive_op(not_implemented_op, [], line)
+    not_implemented = builder.add(LoadAddress(not_implemented_op.type,
+                                              not_implemented_op.src, line))
     builder.add(Branch(
         builder.binary_op(eqval, not_implemented, 'is', line),
         not_implemented_block,
@@ -463,16 +464,15 @@ def create_mypyc_attrs_tuple(builder: IRBuilder, ir: ClassIR, line: int) -> Valu
     attrs = [name for ancestor in ir.mro for name in ancestor.attributes]
     if ir.inherits_python:
         attrs.append('__dict__')
-    return builder.primitive_op(new_tuple_op,
-                             [builder.load_static_unicode(attr) for attr in attrs],
-                             line)
+    items = [builder.load_static_unicode(attr) for attr in attrs]
+    return builder.new_tuple(items, line)
 
 
 def finish_non_ext_dict(builder: IRBuilder, non_ext: NonExtClassInfo, line: int) -> None:
     # Add __annotations__ to the class dict.
-    builder.primitive_op(dict_set_item_op,
-                      [non_ext.dict, builder.load_static_unicode('__annotations__'),
-                       non_ext.anns], -1)
+    builder.call_c(dict_set_item_op,
+                [non_ext.dict, builder.load_static_unicode('__annotations__'),
+                non_ext.anns], -1)
 
     # We add a __doc__ attribute so if the non-extension class is decorated with the
     # dataclass decorator, dataclass will not try to look for __text_signature__.
@@ -505,7 +505,7 @@ def dataclass_finalize(
     """
     finish_non_ext_dict(builder, non_ext, cdef.line)
     dec = builder.accept(next(d for d in cdef.decorators if is_dataclass_decorator(d)))
-    builder.primitive_op(
+    builder.call_c(
         dataclass_sleight_of_hand, [dec, type_obj, non_ext.dict, non_ext.anns], cdef.line)
 
 
@@ -518,10 +518,10 @@ def dataclass_non_ext_info(builder: IRBuilder, cdef: ClassDef) -> Optional[NonEx
     """
     if is_dataclass(cdef):
         return NonExtClassInfo(
-            builder.primitive_op(new_dict_op, [], cdef.line),
+            builder.call_c(dict_new_op, [], cdef.line),
             builder.add(TupleSet([], cdef.line)),
-            builder.primitive_op(new_dict_op, [], cdef.line),
-            builder.primitive_op(type_object_op, [], cdef.line),
+            builder.call_c(dict_new_op, [], cdef.line),
+            builder.add(LoadAddress(type_object_op.type, type_object_op.src, cdef.line))
         )
     else:
         return None
