@@ -93,10 +93,10 @@ def transform_decorator(builder: IRBuilder, dec: Decorator) -> None:
         decorated_func = load_decorated_func(builder, dec.func, orig_func)
 
         # Set the callable object representing the decorated function as a global.
-        builder.primitive_op(dict_set_item_op,
-                          [builder.load_globals_dict(),
-                           builder.load_static_unicode(dec.func.name), decorated_func],
-                          decorated_func.line)
+        builder.call_c(dict_set_item_op,
+                    [builder.load_globals_dict(),
+                    builder.load_static_unicode(dec.func.name), decorated_func],
+                    decorated_func.line)
 
     builder.functions.append(func_ir)
 
@@ -350,13 +350,13 @@ def handle_ext_method(builder: IRBuilder, cdef: ClassDef, fdef: FuncDef) -> None
 
         # Set the callable object representing the decorated method as an attribute of the
         # extension class.
-        builder.primitive_op(py_setattr_op,
-                          [
-                              typ,
-                              builder.load_static_unicode(name),
-                              decorated_func
-                          ],
-                          fdef.line)
+        builder.call_c(py_setattr_op,
+                    [
+                        typ,
+                        builder.load_static_unicode(name),
+                        decorated_func
+                    ],
+                    fdef.line)
 
     if fdef.is_property:
         # If there is a property setter, it will be processed after the getter,
@@ -491,20 +491,20 @@ def handle_yield_from_and_await(builder: IRBuilder, o: Union[YieldFromExpr, Awai
     received_reg = builder.alloc_temp(object_rprimitive)
 
     if isinstance(o, YieldFromExpr):
-        iter_val = builder.primitive_op(iter_op, [builder.accept(o.expr)], o.line)
+        iter_val = builder.call_c(iter_op, [builder.accept(o.expr)], o.line)
     else:
-        iter_val = builder.primitive_op(coro_op, [builder.accept(o.expr)], o.line)
+        iter_val = builder.call_c(coro_op, [builder.accept(o.expr)], o.line)
 
     iter_reg = builder.maybe_spill_assignable(iter_val)
 
     stop_block, main_block, done_block = BasicBlock(), BasicBlock(), BasicBlock()
-    _y_init = builder.primitive_op(next_raw_op, [builder.read(iter_reg)], o.line)
+    _y_init = builder.call_c(next_raw_op, [builder.read(iter_reg)], o.line)
     builder.add(Branch(_y_init, stop_block, main_block, Branch.IS_ERROR))
 
     # Try extracting a return value from a StopIteration and return it.
     # If it wasn't, this reraises the exception.
     builder.activate_block(stop_block)
-    builder.assign(result, builder.primitive_op(check_stop_op, [], o.line), o.line)
+    builder.assign(result, builder.call_c(check_stop_op, [], o.line), o.line)
     builder.goto(done_block)
 
     builder.activate_block(main_block)
@@ -543,7 +543,7 @@ def handle_yield_from_and_await(builder: IRBuilder, o: Union[YieldFromExpr, Awai
     def else_body() -> None:
         # Do a next() or a .send(). It will return NULL on exception
         # but it won't automatically propagate.
-        _y = builder.primitive_op(
+        _y = builder.call_c(
             send_op, [builder.read(iter_reg), builder.read(received_reg)], o.line
         )
         ok, stop = BasicBlock(), BasicBlock()
@@ -557,7 +557,7 @@ def handle_yield_from_and_await(builder: IRBuilder, o: Union[YieldFromExpr, Awai
         # Try extracting a return value from a StopIteration and return it.
         # If it wasn't, this rereaises the exception.
         builder.activate_block(stop)
-        builder.assign(result, builder.primitive_op(check_stop_op, [], o.line), o.line)
+        builder.assign(result, builder.call_c(check_stop_op, [], o.line), o.line)
         builder.nonlocal_control[-1].gen_break(builder, o.line)
 
     builder.push_loop_stack(loop_block, done_block)
