@@ -2,15 +2,17 @@ import unittest
 
 from mypy.nodes import Var
 
-from mypyc.emit import Emitter, EmitterContext
-from mypyc.ops import BasicBlock, Environment, int_rprimitive
+from mypyc.codegen.emit import Emitter, EmitterContext
+from mypyc.ir.ops import BasicBlock, Environment
+from mypyc.ir.rtypes import int_rprimitive
+from mypyc.namegen import NameGenerator
 
 
 class TestEmitter(unittest.TestCase):
     def setUp(self) -> None:
         self.env = Environment()
         self.n = self.env.add_local(Var('n'), int_rprimitive)
-        self.context = EmitterContext(['mod'])
+        self.context = EmitterContext(NameGenerator([['mod']]))
         self.emitter = Emitter(self.context, self.env)
 
     def test_label(self) -> None:
